#version 430

//Images and mipmap levels
layout(binding=0,rgba16f) uniform readonly image2D input_fullSize;
layout(binding=1) uniform sampler2D input_halfSize;
layout(binding=2,rgba16f) uniform writeonly image2D output_fullSize;

uniform ivec2 imageBounds;
uniform vec2 invImageBounds;
uniform float alpha;
uniform float beta;

ivec2 clampLocation(ivec2 xy){
    return clamp(xy,ivec2(0,0),imageBounds.xy);
}

layout (local_size_x = 16, local_size_y = 16) in;

void main() {
	const ivec2 global_xy = ivec2(gl_GlobalInvocationID);
    const ivec2 tile_xy = ivec2(gl_WorkGroupID);
    const ivec2 thread_xy = ivec2(gl_LocalInvocationID);
	const ivec2 pixel_xy = clampLocation(tile_xy*16+thread_xy);

	vec4 one = imageLoad(input_fullSize,pixel_xy);

	vec4 two = textureLod(input_halfSize,(pixel_xy+vec2(1,1))*invImageBounds,0.0f);
	vec4 result = alpha*one+beta*two;
	imageStore(output_fullSize,pixel_xy,vec4(result.rgb,1.0));
}