#version 430

//Blur constants
const int TILE_WIDTH = 16;
const int TILE_HEIGHT = 16;
const ivec2 TILE_SIZE = ivec2(TILE_WIDTH,TILE_HEIGHT);

//Images and mipmap levels
layout(binding=0) uniform sampler2D input_image;
layout(binding=1,rgba16f) uniform writeonly image2D output_image2;
layout(binding=2,rgba16f) uniform writeonly image2D output_image4;
layout(binding=3,rgba16f) uniform writeonly image2D output_image8;
layout(binding=4,rgba16f) uniform writeonly image2D output_image16;
layout(binding=5,rgba16f) uniform writeonly image2D avg_luminance;

//Control parameters
uniform float treshhold;
uniform ivec2 imageBounds;
uniform vec2 invImageBounds;


const vec4 luminance = vec4(0.2126, 0.7152, 0.0722,0.0);

void retirePhase(){
    memoryBarrierShared();
    barrier();
}

ivec2 clampLocation(ivec2 xy){
    return clamp(xy,ivec2(0,0),imageBounds.xy);
}

vec4 brightAndLuminance(vec2 samplePoint){
    vec4 rgba = textureLod(input_image,samplePoint,0.0f);
    //rgba.a = log(max(0,dot(rgba,luminance))+1);
	rgba.a = max(0,dot(rgba,luminance));
    //rgba.rgb = max(rgba.rgb-treshhold,0.0);
	float brightness = dot(rgba,luminance)+0.0001;
	float l = length(max(rgba.rgb-treshhold,vec3(0)));
	//rgba.rgb = max(rgba.rgb*l/brightness,vec3(0));
	rgba.rgb = min(max(normalize(rgba.rgb)*l,vec3(0)),vec3(5));
    return rgba;//vec4(rgba.rgb,1.0);
}

layout (local_size_x = 16, local_size_y = 16) in;

shared vec4 pixel0[16][16];
shared vec4 pixel1[16][16];

void main() {
	const ivec2 global_xy = ivec2(gl_GlobalInvocationID);
    const ivec2 tile_xy = ivec2(gl_WorkGroupID);
    const ivec2 thread_xy = ivec2(gl_LocalInvocationID);
	const ivec2 pixel_xy = clampLocation(global_xy);

	vec2 sampleBetween4Pixel = (2*pixel_xy+vec2(1,1))*invImageBounds;	
	vec4 bright = brightAndLuminance(sampleBetween4Pixel);
	pixel0[thread_xy.x][thread_xy.y] = bright;
	imageStore(output_image2,pixel_xy,bright); // output_image2

    retirePhase();
    if(thread_xy.x < 8 && thread_xy.y < 8){ // output_image4
        vec4 nextLevel = pixel0[thread_xy.x*2][thread_xy.y*2];
        nextLevel += pixel0[thread_xy.x*2+1][thread_xy.y*2];
        nextLevel += pixel0[thread_xy.x*2][thread_xy.y*2+1];
        nextLevel += pixel0[thread_xy.x*2+1][thread_xy.y*2+1];
        nextLevel = nextLevel/4;
        pixel1[thread_xy.x][thread_xy.y] = nextLevel;
        imageStore(output_image4,clamp(tile_xy*8+thread_xy,ivec2(0,0),imageBounds.xy/2),nextLevel);
    }

	retirePhase();
    if(thread_xy.x < 4 && thread_xy.y < 4){ // output_image8
        vec4 nextLevel = pixel1[thread_xy.x*2][thread_xy.y*2];
        nextLevel += pixel1[thread_xy.x*2+1][thread_xy.y*2];
        nextLevel += pixel1[thread_xy.x*2][thread_xy.y*2+1];
        nextLevel += pixel1[thread_xy.x*2+1][thread_xy.y*2+1];
        nextLevel = nextLevel/4.0;
        pixel0[thread_xy.x*2][thread_xy.y*2] = nextLevel;
        imageStore(output_image8,clamp(tile_xy*4+thread_xy,ivec2(0,0),imageBounds.xy/4),nextLevel);
    }
	retirePhase();
    if(thread_xy.x < 2 && thread_xy.y < 2){// output_image16
        vec4 nextLevel = pixel0[thread_xy.x*2][thread_xy.y*2];
        nextLevel += pixel0[thread_xy.x*2+1][thread_xy.y*2];
        nextLevel += pixel0[thread_xy.x*2][thread_xy.y*2+1];
        nextLevel += pixel0[thread_xy.x*2+1][thread_xy.y*2+1];
        nextLevel = nextLevel/4.0;
        imageStore(output_image16,clamp(tile_xy*2+thread_xy,ivec2(0,0),imageBounds.xy/8),nextLevel);
		imageStore(avg_luminance,clamp(tile_xy*2+thread_xy,ivec2(0,0),imageBounds.xy/8),nextLevel);
    }
	 
	//retirePhase();
    //const vec4 color = vec4(max(rgb-vec3(treshhold,treshhold,treshhold),vec3(0.f)), 1.f);
    //smoothstep(treshhold, treshhold*1.5f, brightness) * vec4(rgb, 1.0f);
	//const float eitherOr = clamp(sign(brightness - treshhold), 0.0f, 1.0f);
	//const vec4 color = mix(vec4(0.0f),vec4(rgb,1.0f),eitherOr);	
}