#version 430 core

layout(location=0) in vec4 position_invMass;
layout(location=1) in vec4 prevPosition;

uniform samplerBuffer position_invMassMap; // slot 0
uniform samplerBuffer prevPositionMap; // slot 1

uniform vec2 cellSize;
uniform ivec2 gridSize;

uniform vec3 springForces_damping; // spring stiffness, spring damping, total damping
uniform vec3 gravity;
uniform float dt;

out vec4 outPosition_invMass;

ivec2 offsets[12] = ivec2[](
	ivec2(1,0), ivec2(-1,0), ivec2(0,1), ivec2(0,-1),
	ivec2(-1,-1), ivec2(-1,1), ivec2(1,-1), ivec2(1,1),
	ivec2(-2,0), ivec2(2,0), ivec2(0,2), ivec2(0,-2)
);

float restLengthFactors[12] = float[](
	1.f, 1.f, 1.f, 1.f,
	1.414f, 1.414f, 1.414f, 1.414f,
	2.f, 2.f, 2.f, 2.f
);

void main(void)
{
	vec3 p = position_invMass.xyz;
	vec3 pOld = prevPosition.xyz;
	vec3 v = (p - pOld) / dt;
	float invM = position_invMass.w;

	vec3 f = v * springForces_damping.z;

	int ix = gl_VertexID % gridSize.x;
	int iy = gl_VertexID / gridSize.x;

	int ixN, iyN;
	for (int i=0;i<12;i++)
	{
		ixN = ix + offsets[i].x;
		iyN = iy + offsets[i].y;

		bool inBounds = (ixN >= 0 && ixN < gridSize.x && iyN >= 0 && iyN < gridSize.y);
		int index = iyN * gridSize.x + ixN;

		vec3 pN = texelFetch(position_invMassMap, index).xyz;
		vec3 pOldN = texelFetch(prevPositionMap, index).xyz;

		float restLength = restLengthFactors[i] * cellSize.x;

		vec3 vN = (pN - pOldN) / dt;
		vec3 dP = p - pN;
		vec3 dV = v - vN;
		float dist = length(dP);
		vec3 dP_norm = dP / dist;

		float stiffnessTerm = -springForces_damping.x * (dist - restLength);
		float dampingTerm = springForces_damping.y * (dot(dV, dP_norm));
		vec3 springForce = (stiffnessTerm + dampingTerm) * dP_norm;
		f += mix(vec3(0.f), springForce, inBounds);
	}

	vec3 a = f * invM;
	a += mix(vec3(0.f), gravity, invM > 0.f);

	vec3 temp = p;
	vec3 pNew = 2.f * p - pOld + a*dt*dt;

	outPosition_invMass = vec4(pNew, invM);
	gl_Position = vec4(p, 1.f);
}