#version 430 core

uniform sampler2D colorMap; // 0
uniform sampler2D dof_depthMap; // 1

uniform float focalPlaneDepth;

subroutine void passType();
layout(location=0) subroutine uniform passType pass;

out vec3 outColor;

void dof(ivec2 direction)
{
	ivec2 fragCoord = ivec2(gl_FragCoord.xy);
	vec3 colorCenter = texelFetch(colorMap, fragCoord, 0).xyz;
	vec4 dof_depthCenter = texelFetch(dof_depthMap, fragCoord, 0); // CoC, 1/CoC^2, ---, viewPosition.z

	float totalWeight = dof_depthCenter.y;
	vec3 totalColor = totalWeight * colorCenter;
	//float totalWeight = 0.f;
	//vec3 totalColor = vec3(0.f,0.f,0.f);

	if (dof_depthCenter.x <= 1.f)
	{
		outColor = totalColor;
		return;
	}

	for(int i=-15; i<15; i++)
	{
		if (i == 0)
			continue;

		ivec2 pixelCoord = fragCoord + direction * ivec2(i, i);
		vec3 colorPixel = texelFetch(colorMap, pixelCoord, 0).xyz;
		vec4 dof_depthPixel = texelFetch(dof_depthMap, pixelCoord, 0);

		float overlap = 1.f - smoothstep(0,dof_depthPixel.x,abs(i));//clamp(dof_depthPixel.x - abs(i), 0.f, 1.f);
		float intensity = dof_depthPixel.y;
		float leakage = min(dof_depthCenter.x, 1.f);//(-dof_depthPixel.w > focalPlaneDepth) ?  : 1.f;

		float weight = overlap * intensity * leakage;

		totalWeight += weight;
		totalColor += weight * colorPixel;
	}

	totalColor /= totalWeight;

	outColor = totalColor;
}

subroutine(passType)
void pass1()
{
	dof(ivec2(0,1));
}

subroutine(passType)
void pass2()
{
	dof(ivec2(1,0));
}

void main(void)
{
	pass();
}