#version 430 core

in vec4 vf_normal_linearDepth;
in vec3 vf_tangent;
in vec3 vf_bitangent;
in vec2 vf_texCoords;
flat in uint vf_drawID;

uniform sampler2D normalMap; // slot 0

// shininess should be normalized [0,255]->[0,1]
// objectID should be normalized [0,32767]->[0,1]
//uniform vec3 shininess_objectID_normalMapStrength;

struct MeshInstanceProperties
{
	mat4 worldViewProjectionMatrix;
	mat3 normalMatrix;
	vec4 worldViewMatrix_row3;
	vec3 shininess_objectID_normalMapStrength;
};

layout(std430, binding=0) buffer MeshInstancePropertiesBlock
{
	MeshInstanceProperties prop[];
};

subroutine void fetchNormalType();
layout(location=0) subroutine uniform fetchNormalType fetchNormal;

layout(location=0) out vec3 outInterpolatedNormal; // these values are normalized before saving [-1,1] -> [0,1]
layout(location=1) out float outShininess; // value has been normalized [0,255]->[0,1]
layout(location=2) out float outID; // value has been normalized [0,32767]->[0,1]
layout(location=3) out float outLinearDepth;

subroutine(fetchNormalType)
void fetchNormalWithNormalMapping()
{
	mat3 tangentToViewSpace = mat3(
		normalize(vf_tangent),
		normalize(vf_bitangent),
		normalize(vf_normal_linearDepth.xyz)
	);
	vec3 normalInTangentSpace = texture(normalMap, vf_texCoords).xyz * 2.f - 1.f;
	float normalMapStrength = prop[vf_drawID].shininess_objectID_normalMapStrength.z;
	outInterpolatedNormal =	0.5f + 0.5f * mix(-1.f, 1.f, gl_FrontFacing) * (tangentToViewSpace * normalize(vec3(normalMapStrength,normalMapStrength,1.f) * normalInTangentSpace));
}

subroutine(fetchNormalType)
void fetchNormalWithoutNormalMapping()
{
	outInterpolatedNormal = 0.5f + 0.5f * mix(-1.f, 1.f, gl_FrontFacing) * normalize(vf_normal_linearDepth.xyz);
}

void main(void)
{
	fetchNormal();
	outShininess = prop[vf_drawID].shininess_objectID_normalMapStrength.x;
	outID = prop[vf_drawID].shininess_objectID_normalMapStrength.y;
	outLinearDepth = vf_normal_linearDepth.w;
}