#version 430

//Blur constants
const int TILE_WIDTH = 16;
const int TILE_HEIGHT = 16;
const ivec2 TILE_SIZE = ivec2(TILE_WIDTH,TILE_HEIGHT);

layout(binding=0,rgba16f) uniform image2D input_image;
layout(binding=1,rgba16f) uniform image2D output_image;

uniform float treshhold;
uniform ivec4 imageBounds;
//Size of the input image

void retirePhase(){
    memoryBarrierShared();
    barrier();
}

ivec2 clampLocation(ivec2 xy){
    return clamp(xy,imageBounds.xy,imageBounds.zw);
}

layout (local_size_x = 16, local_size_y = 16) in;

void main() {
	const ivec2 tile_xy = ivec2(gl_WorkGroupID);
    const ivec2 thread_xy = ivec2(gl_LocalInvocationID);
    const ivec2 pixel_xy = tile_xy * TILE_SIZE + thread_xy;
	
	if(pixel_xy.x < imageBounds.z && pixel_xy.y < imageBounds.w){
		vec4 rgba = imageLoad(input_image,pixel_xy);

		//weighted quadratic distance, TODO: try cheaper brightness calculations (there are many)
		//RGBA in 0-1.0f space
		float brightness = sqrt(rgba.r * rgba.r * .241 + rgba.g * rgba.g * .691 + rgba.b * rgba.b * .068);

		if(brightness >= treshhold){
			imageStore(output_image,pixel_xy,rgba);
		}else{
			imageStore(output_image,pixel_xy,vec4(0.0f,0.0f,0.0f,1.0f));
		}	
	}
	
}