#version 430 core

const float distortionStrength = 1.f;

uniform sampler2D texture0;
uniform sampler2D lensColorTexture;
uniform vec2 invImageSize;

in vec2 vf_texCoords;

layout(location=0) out vec3 outColor;

vec3 distortedTexture(in vec2 texCoords, in vec2 distortion)
{
	return vec3(
		texture(texture0, texCoords + distortion).r,
		texture(texture0, texCoords).g,
		texture(texture0, texCoords - distortion).b
	);
}

void main(void)
{
	vec2 mirroredTexCoords = -vf_texCoords + 1.0f;

	vec2 ghostVector = (0.5f - mirroredTexCoords) * 0.235746f;
	vec2 direction = normalize(ghostVector);

	vec2 distortionR = -direction * invImageSize.x * distortionStrength;

	vec3 result = vec3(0.f);
	
	for(int i=0;i<8;i++)
	{
		vec2 offset = fract(mirroredTexCoords + ghostVector * float(i));

		float weight = length(0.5f - offset) / length(vec2(0.5f));
		weight = pow(1.0f - weight, 10.0f);

		result += distortedTexture(offset, distortionR) * weight;
	}

	float lensColorCoord = length(0.5f - mirroredTexCoords) / length(vec2(0.5f));
	result *= texture(lensColorTexture, vec2(lensColorCoord, 0.5f)).rgb;

	
	vec2 haloVector = direction * 0.5f;
	float haloWeight = length(0.5f - fract(mirroredTexCoords + haloVector)) / length(vec2(0.5f));
	haloWeight = pow(1.0 - haloWeight, 10.0f);
	result += distortedTexture(mirroredTexCoords + haloVector, distortionR)  * haloWeight;
	

	outColor = result;
}