#version 430

//Blur constants
const int TILE_WIDTH = 16;
const int TILE_HEIGHT = 16;
const ivec2 TILE_SIZE = ivec2(TILE_WIDTH,TILE_HEIGHT);

const float[] gaussianKernel = {0.161964,0.217820,0.240432,0.217820,0.161964};
const int FILTER_WIDTH = 5;
const int FILTER_HEIGHT = 5;
const ivec2 FILTER_OFFSET = ivec2(FILTER_WIDTH/2,FILTER_HEIGHT/2);

const ivec2 NEIGHBOURHOOD_SIZE = TILE_SIZE + 2*FILTER_OFFSET;	
const int NEIGHBORHOOD_WIDTH = NEIGHBOURHOOD_SIZE.x;
const int NEIGHBORHOOD_HEIGHT = NEIGHBOURHOOD_SIZE.y;

layout(binding=0,rgba16f) uniform image2D input_image;
layout(binding=1,rgba16f) uniform image2D output_image;

uniform ivec4 imageBounds;
//Size of the input image

void retirePhase(){
    memoryBarrierShared();
    barrier();
}

ivec2 clampLocation(ivec2 xy){
    return clamp(xy,imageBounds.xy,imageBounds.zw);
}

//Layout(TILE_WIDTH,NEIGHBORHOOD_HEIGHT);
layout (local_size_x = 16, local_size_y = 20) in;
//20x20  pixel[NEIGHBOURHOOD_HEIGHT][NEIGHBOURHOOD_WIDTH]
shared vec4 pixel[20][20];//values read from input image
//20x16   row[NEIGHBOURHOOD_HEIGHT][TILE_WIDTH]
shared vec4 row[20][16];//weighted row sums

void main() {
	const ivec2 tile_xy = ivec2(gl_WorkGroupID); //0,0
    const ivec2 thread_xy = ivec2(gl_LocalInvocationID);//0,0
    const ivec2 pixel_xy = tile_xy * TILE_SIZE + (thread_xy - ivec2(0, FILTER_OFFSET.y));//0,-2
    const uint x = thread_xy.x;
    const uint y = thread_xy.y;

    //READ into shared pixel array
	//Todo add treshhold filtering
	//The for loop is here to cover the 16-19 pixels in x direction as the code is optimized in terms of number of threads for separated kernel
	//i=0,i=16
	for(int i=0; i < NEIGHBORHOOD_WIDTH; i += TILE_HEIGHT){
			//i=0 :	x+i=x=0..15
			//i=16:	x+i=16..19
            if(x+i < NEIGHBORHOOD_WIDTH){
                const ivec2	 read_at = clampLocation(pixel_xy + ivec2(i-FILTER_OFFSET.x,0));//(-2,-2)=> (0,0)
                pixel[y][x+i] = imageLoad(input_image,read_at);//read(0,0),read(0,16)					
            }
	}
	retirePhase();
    
	//HORIZONTAL GAUSSIAN BLUR
	row[y][x] = vec4(0.0f);
    for(int i=0; i<FILTER_WIDTH; i++){
        row[y][x] += pixel[y][x+i] * gaussianKernel[i];
	}
	retirePhase();
	
	//VERTICAL GAUSSIAN BLUR
	
	//Phase 3: Weight sum vertically and write to output image
	//Does this thread corrsepond to a tile pixel?
	//Recall there are more threads in y direction than tileheight

	if(y < TILE_HEIGHT){
		vec4 result = vec4(0.0f);
		for(int i=0;i < FILTER_HEIGHT;i++){
			result += row[y+i][x] * gaussianKernel[i];
		}

		//Store the result
		const ivec2 inputPosition = tile_xy*TILE_SIZE + thread_xy;
		//const vec4 inValue = imageLoad(input_image,inputPosition);

		imageStore(output_image,inputPosition,result);
	}
}