#version 430 core

noperspective in vec2 vf_ndcCoordinates;

uniform sampler2D normalMap; //slot 0, these values are normalized from [-1,1] -> [0,1]
uniform sampler2D depthMap; //slot 1, stores the z-coordinate of the view position (is always negative)
uniform sampler2D shininessMap; // slot 2

// stay constant over longer period of time
uniform vec2 projectionEntries;

// different for each light
uniform vec3 lightViewNegatedDirection;
uniform vec3 lightColor;

layout(location=0) out vec4 accumulatedLight;

#include <lum.glsl>

void main(void)
{
	ivec2 screenCoords = ivec2(gl_FragCoord.xy);
	
	// view position from depth and screen position
	vec3 surfaceViewPosition;
	surfaceViewPosition.z = texelFetch(depthMap, screenCoords, 0).x;
	surfaceViewPosition.xy = -vf_ndcCoordinates * surfaceViewPosition.z / projectionEntries.xy;

	vec3 l = lightViewNegatedDirection; // from surface point to light source

	// get variables
	vec3 n = 2.f * texelFetch(normalMap, screenCoords, 0).xyz - 1.f;
	vec3 v = normalize(surfaceViewPosition); // from viewer to surface point (negated view vector)

	// diffuse
	float cosTheta = max(0.f, dot(n,l));

	// specular
	float shininess = texelFetch(shininessMap, screenCoords, 0).x * 255.f;
	vec3 r = reflect(l,n); // negated reflection vector
	float specular = max(0.f, pow(dot(v,r), shininess)); // both v and r are negated -> dot product is correct

	vec3 attenuatedColor = lightColor * cosTheta;
	accumulatedLight.xyz = attenuatedColor;
	accumulatedLight.w = lum(attenuatedColor * specular);

}