#version 430 core

noperspective in vec2 vf_ndcCoordinates;
flat in int vf_instanceID;

uniform sampler2D normalMap; //slot 0, these values are normalized from [-1,1] -> [0,1]
uniform sampler2D depthMap; //slot 1, stores the z-coordinate of the view position (is always negative)
uniform sampler2D shininessMap; // slot 2

// stay constant over longer period of time
uniform vec2 projectionEntries;

// different for each light
struct PointLightProperties
{
	vec3 lightViewPosition;
	vec3 lightColor;
	vec2 lightAttenuation_radius; // 1/(range^2), range
};

layout(std140, binding=1) uniform PointLightPropertiesBlock {
	PointLightProperties prop[100];
};

layout(location=0) out vec4 accumulatedLight;

#include <lum.glsl>

void main(void)
{
	ivec2 screenCoords = ivec2(gl_FragCoord.xy);
	
	// view position from depth and screen position
	vec3 surfaceViewPosition;
	surfaceViewPosition.z = texelFetch(depthMap, screenCoords, 0).x;
	surfaceViewPosition.xy = -vf_ndcCoordinates * surfaceViewPosition.z / projectionEntries.xy;

	vec3 l = prop[vf_instanceID].lightViewPosition - surfaceViewPosition; // from surface point to light source
	float dist = length(l);

	if (dist < prop[vf_instanceID].lightAttenuation_radius.y)
	{
		// get variables
		l = l / dist;
		vec3 n = 2.f * texelFetch(normalMap, screenCoords, 0).xyz - 1.f;
		vec3 v = normalize(surfaceViewPosition); // from viewer to surface point (negated view vector)

		// diffuse
		float cosTheta = max(0.f, dot(n,l));

		// specular
		float shininess = texelFetch(shininessMap, screenCoords, 0).x * 255.f;
		vec3 r = reflect(l,n); // negated reflection vector
		float specular = max(0.f, pow(dot(v,r), shininess)); // both v and r are negated -> dot product is correct

		// attenuate and store
		float attenuation = 1.f - (dist*dist)*prop[vf_instanceID].lightAttenuation_radius.x;
		vec3 attenuatedColor = prop[vf_instanceID].lightColor * attenuation * attenuation * cosTheta;

		accumulatedLight.xyz = attenuatedColor;
		accumulatedLight.w = lum(attenuatedColor * specular);
	}
	else
	{
		accumulatedLight = vec4(0.f,0.f,0.f,0.f);
	}
}