#version 430 core

layout (local_size_x = 512, local_size_y = 1, local_size_z = 1) in;

uniform uint count;
uniform vec4 gravityTimesDt_dt;
uniform vec4 sphereColliders[%numSphereColliders%]; //xyz=position, w=radius
uniform vec4 boxColliders[%numBoxColliders%*2];

struct CollisionConstraint
{
	vec4 q_active;
	vec4 n; //w inactive
};

layout(std430, binding=0) buffer velocities {
	vec3 v[];
};

layout(std430, binding=1) buffer positionEstimates {
	vec4 p[];
};

layout(std430, binding=2) buffer positions {
	vec3 x[];
};

layout(std430, binding=5) buffer collisionConstraints {
	CollisionConstraint constraint[];
};

void main(void)
{
	uint gid = gl_GlobalInvocationID.x;
	if (gid < count)
	{
		float pw = p[gid].w;
		vec3 newVel = 0.999f*(v[gid] + gravityTimesDt_dt.xyz);
		vec3 newPosEstimate = x[gid] + newVel * gravityTimesDt_dt.w * pw;
		v[gid] = newVel;
		p[gid] = vec4(newPosEstimate, pw);

		// generate collision constraints
		vec4 newQ_active = vec4(0.f,0.f,0.f,0.f);
		for(int i=0; i<%numSphereColliders%; i++)
		{
			vec3 centerToPosition = newPosEstimate - sphereColliders[i].xyz;
			if (dot(centerToPosition,centerToPosition) < sphereColliders[i].w*sphereColliders[i].w)
			{
				vec3 sphereNormal = normalize(centerToPosition);
				newQ_active = vec4(sphereColliders[i].xyz + sphereNormal*sphereColliders[i].w, 1.0f);
				constraint[gid*2].n = vec4(sphereNormal,0.f);
			}
		}
		constraint[gid*2].q_active = newQ_active;

		newQ_active = vec4(0.f,0.f,0.f,0.f);
		for(int i=0; i<%numBoxColliders%; i++)
		{
			vec3 minPoint = boxColliders[2*i].xyz;
			vec3 maxPoint = boxColliders[2*i+1].xyz;
			bool inside = all(greaterThan(newPosEstimate, minPoint)) && all(lessThan(newPosEstimate, maxPoint));
			if (inside)
			{
				vec3 distances = newPosEstimate - minPoint;
				float minDist = min(distances.x, min(distances.y, distances.z));
				bvec3 minComp = equal(distances, vec3(minDist));
				vec3 normal = mix(vec3(0.f,0.f,0.f), vec3(-1.f,-1.f,-1.f), minComp);
				vec3 q = mix(newPosEstimate, minPoint, minComp);

				distances = maxPoint - newPosEstimate;
				float minDist2 = min(distances.x, min(distances.y, distances.z));
				minComp = equal(distances, vec3(minDist2));
				vec3 normal2 = mix(vec3(0.f,0.f,0.f), vec3(1.f,1.f,1.f), minComp);
				vec3 q2 = mix(newPosEstimate, maxPoint, minComp);

				bool secondLarger = minDist2 > minDist;
				normal = mix(normal2, normal, secondLarger);
				q = mix(q2, q, secondLarger);

				constraint[gid*2+1].n = vec4(normal, 0.f);
				newQ_active = vec4(q, 1.f);
			}
		}
		constraint[gid*2+1].q_active = newQ_active;
	}
}