#version 430 core

layout (local_size_x = 512, local_size_y = 1, local_size_z = 1) in;

uniform uint numColors;
uniform float stiffness;
uniform uint constraintsPerColor[64];

struct DistanceConstraint {
	uint i1;
	uint i2;
	float d;
};

layout(std430, binding=0) buffer velocities {
	vec3 v[];
};

layout(std430, binding=1) buffer positionEstimates {
	coherent vec4 p[];
};

layout(std430, binding=2) buffer positions {
	vec3 x[];
};

layout(std430, binding=3) buffer distanceConstraints {
	DistanceConstraint constraint[];
};

layout(std430, binding=4) buffer weights {
	float weight[];
};

void main(void)
{
	uint gid = gl_GlobalInvocationID.x;
	uint offset = gid;

	for (uint i=0; i<numColors; i++)
	{
		uint numConstraints = constraintsPerColor[i];
		if (gid < numConstraints)
		{
			uint i1 = constraint[offset].i1;
			uint i2 = constraint[offset].i2;
			float d = constraint[offset].d;

			vec3 p1 = p[i1].xyz;
			vec3 p2 = p[i2].xyz;
			float w1 = weight[i1];
			float w2 = weight[i2];

			float dist = distance(p1,p2);
			float sumWeights = max(w1+w2, 0.000001f);
			vec3 rightTerm = stiffness * (dist - d) * (p1 - p2) / (dist * sumWeights);

			p[i1].xyz += -w1 * rightTerm;
			p[i2].xyz += w2 * rightTerm;
		}
		offset += numConstraints;
		
		memoryBarrierBuffer();
		barrier();
	}
}