#version 430 core

layout (local_size_x = 512, local_size_y = 1, local_size_z = 1) in;

uniform uint count;
uniform float dt;

struct NeighborhoodPointer
{
	uint index;
	uint len;
};

layout(std430, binding=0) buffer velocities {
	vec3 v[];
};

layout(std430, binding=1) buffer positionEstimates {
	vec4 p[];
};

layout(std430, binding=2) buffer positions {
	vec3 x[];
};

layout(std430, binding=3) buffer normals {
	vec3 n[];
};

layout(std430, binding=4) buffer neighborhood {
	uint neighbor[];
};

layout(std430, binding=5) buffer neighborhoodPointers {
	NeighborhoodPointer np[];
};

layout(std430, binding=6) buffer uvs {
	vec2 uv[];
};

layout(std430, binding=7) buffer tangents {
	vec3 t[];
};

layout(std430, binding=8) buffer bitangents {
	vec3 b[];
};



void main(void)
{
	uint gid = gl_GlobalInvocationID.x;
	if (gid < count)
	{
		vec3 pLookup = p[gid].xyz;
		v[gid] = (pLookup - x[gid]) / dt;
		x[gid] = pLookup;

		NeighborhoodPointer pointer = np[gid];
		uint len = pointer.len;
		uint index = pointer.index;

		vec2 uvLookup = uv[gid];

		//n[gid] = vec3(0.f,0.f,0.f);
		//t[gid] = vec3(0.f,0.f,0.f);
		//b[gid] = vec3(0.f,0.f,0.f);

		vec3 N = vec3(0.f,0.f,0.f);
		vec3 T = vec3(0.f,0.f,0.f);
		vec3 B = vec3(0.f,0.f,0.f);
		
		uint firstNeighbor = neighbor[index];
		uint tmpNeighbor = firstNeighbor;
		for (uint i=0; i<len-1; i++)
		{
			vec3 n1pos = p[tmpNeighbor].xyz - pLookup;
			vec2 n1uv = uv[tmpNeighbor] - uvLookup;

			tmpNeighbor = neighbor[index+i+1];
			vec3 n2pos = p[tmpNeighbor].xyz - pLookup;
			vec2 n2uv = uv[tmpNeighbor] - uvLookup;

			N += cross(n1pos, n2pos);

			float r = 1.0f / (n1uv.x * n2uv.y - n1uv.y * n2uv.x);
			T += (n1pos * n2uv.y - n2pos * n1uv.y)*r;
			B += (n2pos * n1uv.x - n1pos * n2uv.x)*r;
		}

		n[gid] = N;
		t[gid] = T;
		b[gid] = B;
	}

	
}