#version 430 core

in vec2 vf_ndcPosition;

uniform sampler2D depthMap; // slot 0
uniform vec2 projectionEntries;
uniform vec2 invImageSize;

layout(location=0) out vec3 perPixelNormal; // these values are normalized before saving [-1,1] -> [0,1]

void main(void)
{
	float viewDepth = texelFetch(depthMap, ivec2(gl_FragCoord.xy), 0).x;
	vec3 viewPosition = vec3(-vf_ndcPosition * viewDepth / projectionEntries.xy, viewDepth);

	// reconstruct view surface normal
	float depthLeft = texelFetch(depthMap, ivec2(gl_FragCoord.x-1.f, gl_FragCoord.y), 0).x;
	float depthRight = texelFetch(depthMap, ivec2(gl_FragCoord.x+1.f, gl_FragCoord.y), 0).x;
	bool useLeft = abs(depthLeft-viewDepth) < abs(depthRight-viewDepth);
	vec2 depthMultiplierH = mix(vec2(depthRight,-1.f), vec2(depthLeft,1.f), useLeft);
	vec3 viewPositionH = depthMultiplierH.y * (viewPosition - vec3(- (vf_ndcPosition - vec2(depthMultiplierH.y*2.f*invImageSize.x,0.f))
		* depthMultiplierH.x / projectionEntries.xy, depthMultiplierH.x));

	float depthBottom = texelFetch(depthMap, ivec2(gl_FragCoord.x, gl_FragCoord.y-1.f), 0).x;
	float depthTop = texelFetch(depthMap, ivec2(gl_FragCoord.x, gl_FragCoord.y+1.f), 0).x;
	bool useBottom = abs(depthBottom-viewDepth) < abs(depthTop-viewDepth);
	vec2 depthMultiplierV = mix(vec2(depthTop, -1.f), vec2(depthBottom,1.f), useBottom);
	vec3 viewPositionV = depthMultiplierV.y * (viewPosition - vec3(- (vf_ndcPosition - vec2(0.f,depthMultiplierV.y*2.f*invImageSize.y))
		* depthMultiplierV.x / projectionEntries.xy, depthMultiplierV.x));

	perPixelNormal = 0.5f + 0.5f * normalize(cross(viewPositionH, viewPositionV));
}