#version 430 core

in vec2 f_texCoords;
in vec3 f_tangentLightVector;
in vec3 f_tangentViewVector;
in float f_paintValue;

layout(binding=0) uniform sampler2D diffuseMap;
layout(binding=1) uniform sampler2D paintMap;
layout(binding=3) uniform sampler2D normalMap;
layout(binding=4) uniform sampler2D stone_diffuseMap;
layout(binding=5) uniform sampler2D stone_normalMap;

uniform float shininess = 5.f;

uniform vec3 lightColor;
uniform vec2 lightAttenuation_radius;

layout(location=0) out vec3 outColor;
layout(location=1) out vec2 velocity;

void main(void)
{
	vec2 transformedUVs = vec2(f_texCoords.x*2.f-0.5f, f_texCoords.y);
	vec3 stoneNormal = texture(stone_normalMap, transformedUVs).xyz  * 2.f - 1.f;
	stoneNormal = vec3(0.3f, 0.3f, 1.f) * stoneNormal;
	vec3 n = normalize(mix(stoneNormal, texture(normalMap, transformedUVs).xyz  * 2.f - 1.f, f_paintValue));
	vec3 l = f_tangentLightVector;
	vec3 v = normalize(f_tangentViewVector);

	float dist = length(l);
	l = l / dist;

	float cosTheta = max(0.f, dot(n,l));

	vec3 r = reflect(l,n);
	float specular = max(0.f, pow(dot(v,r), shininess));

	float attenuation = 1.f - dist*dist*lightAttenuation_radius.x;

	vec3 attenuatedColor = lightColor * attenuation * attenuation * cosTheta;
	vec3 diffuseColor = attenuatedColor * mix(texture(stone_diffuseMap, transformedUVs).xyz, texture(diffuseMap, transformedUVs).xyz, f_paintValue);
	vec3 specularColor = attenuatedColor * specular * 0.5f;

	outColor = (diffuseColor + specularColor);
	velocity = vec2(0.5f,0.5f);
}