#version 430 core

layout(triangles, fractional_odd_spacing, ccw) in;

in vec3 e_position[];
in vec3 e_normal[];
in vec2 e_texCoords[];
in vec3 e_tangentLightVector[];
in vec3 e_tangentViewVector[];

layout(binding=1) uniform sampler2D paintMap;
layout(binding=2) uniform sampler2D heightMap;

uniform mat4 worldViewProjectionMatrix;

out vec2 f_texCoords;
out vec3 f_tangentLightVector;
out vec3 f_tangentViewVector;
out float f_paintValue;

void main(void)
{
	vec3 position = gl_TessCoord.x * e_position[0] + gl_TessCoord.y * e_position[1] + gl_TessCoord.z * e_position[2];
	vec3 normal = gl_TessCoord.x * e_normal[0] + gl_TessCoord.y * e_normal[1] + gl_TessCoord.z * e_normal[2];
	f_texCoords = gl_TessCoord.x * e_texCoords[0] + gl_TessCoord.y * e_texCoords[1] + gl_TessCoord.z * e_texCoords[2];
	f_tangentLightVector = gl_TessCoord.x * e_tangentLightVector[0] + gl_TessCoord.y * e_tangentLightVector[1] + gl_TessCoord.z * e_tangentLightVector[2];
	f_tangentViewVector = gl_TessCoord.x * e_tangentViewVector[0] + gl_TessCoord.y * e_tangentViewVector[1] + gl_TessCoord.z * e_tangentViewVector[2];

	float height = texture(heightMap, vec2(f_texCoords.x*2.f-0.5f, f_texCoords.y)).r;
	f_paintValue = texture(paintMap, f_texCoords).r;
	float displacement = mix(1.f, height, f_paintValue);
	position += normal*displacement*0.2f;
	
	gl_Position = worldViewProjectionMatrix * vec4(position, 1.f);
	
}