#version 330 core

#define RECONSTRUCT_NORMAL 0

in vec2 vf_ndcPosition;

uniform sampler2D depthMap; // slot 0
uniform sampler2D noiseMap; // slot 1
uniform sampler2D perPixelNormalMap; // //slot 2, these values are normalized from [-1,1] -> [0,1]
uniform vec4 projectionEntries;
uniform vec4 invImageSize_imageSize;



uniform vec3 sampleKernel[16];

out vec3 ambientOcclusion;

void main(void)
{
	float viewDepth = texelFetch(depthMap, ivec2(gl_FragCoord.xy), 0).x;
	vec3 viewPosition = vec3(-vf_ndcPosition * viewDepth / projectionEntries.xy, viewDepth);

	vec3 viewNormal = 2.f * texelFetch(perPixelNormalMap, ivec2(gl_FragCoord.xy), 0).xyz - 1.f;
	
	vec3 rvec = texture(noiseMap, gl_FragCoord.xy/4.f).xyz;
	vec3 tangent = normalize(rvec - viewNormal * dot(rvec, viewNormal));
	vec3 bitangent = cross(viewNormal, tangent);
	mat3 tbn = mat3(tangent, bitangent, viewNormal);

	float accessibility = 0.0f;
	const float radius = 1.f;

	for (int i=0; i<16; i++)
	{
		vec3 viewSamplePosition = tbn * (sampleKernel[i] * 0.05f * (-viewDepth));
		viewSamplePosition = viewSamplePosition * radius + viewPosition; //viewSamplePosition.z depth in half sphere
		
		vec2 sampleTexCoords = viewSamplePosition.xy * projectionEntries.xy / (-viewSamplePosition.z);
		sampleTexCoords = 0.5f * sampleTexCoords + 0.5f;

		float depthS = -texelFetch(depthMap, ivec2(round(sampleTexCoords * invImageSize_imageSize.zw)), 0).x;

		float rangeIsInvalid = smoothstep(depthS, depthS+3.f, -viewDepth);
		accessibility += mix (float(depthS+0.001 > -viewSamplePosition.z), 1.f, rangeIsInvalid);
	}

	float final = accessibility/16.f;
	ambientOcclusion = 0.4f + 0.6f*vec3(final,final,final);
}